<?php

namespace Drp\Analytics\Foundation;

use Drp\Analytics\Contracts\HandlesAnalytics;
use Drp\Analytics\Foundation\Handlers\MySqlAnalytic;
use Drp\Analytics\Foundation\Handlers\MySqlMetric;
use Drp\Analytics\Jobs\StoreAnalytic;
use Illuminate\Support\Arr;
use Illuminate\Support\Traits\Macroable;

class AnalyticsManager
{
    use Macroable;

    public const DRIVER_MYSQL = 'mysql';

    public $handlers = [
        'analytic' => [
            self::DRIVER_MYSQL => MySqlAnalytic::class,
        ],
        'metric' => [
            self::DRIVER_MYSQL => MySqlMetric::class,
        ],
    ];

    public function store(AnalyticReference $reference, $payload): void
    {
        $payload = new AnalyticPayload($payload);

        StoreAnalytic::dispatch($this->handlerFor($reference), $reference, $payload)
            ->onQueue(config('analytics.queue'));
    }

    private function handlerFor(AnalyticReference $reference): HandlesAnalytics
    {
        $type = config("analytics.trackers.{$reference->getType()}");
        $handlerKey = config("analytics.drivers.{$type}");

        $handlerClass = Arr::get($this->handlers, "{$type}.{$handlerKey}");

        return new $handlerClass;
    }

    public function registerHandler(string $type, string $driver, string $handler): void
    {
        Arr::set($this->handlers, "{$type}.{$driver}", $handler);
    }

    public function renderJs(): string
    {
        return view('Analytics::analytics-js')->render();
    }
}
